"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const fs = require("fs");
const path = require("path");
const fse = require("fs-extra");
const templates_1 = require("./templates");
class FileLinker {
    constructor(config) {
        this.templates = templates_1.default;
        this.shouldLink = false;
        const { path = '.', links = {}, templates = {}, extension = 'tsx', indexExtension = 'ts', } = config;
        this.config = _.cloneDeep(config);
        this.path = path;
        this.links = links;
        this.templates = Object.assign({}, this.templates, templates);
        this.paths = [];
        this.files = [];
        this.extension = extension;
        this.indexExtension = indexExtension;
    }
    static get importTemplate() {
        return 'import ${NAME} from \'./${NAME}\'\n';
    }
    static get exportTemplate() {
        return 'export {\n${EXPORTS}}\n';
    }
    static renderImportPart(files = []) {
        let str = '';
        for (const file of files) {
            str += FileLinker.importTemplate.replace(/\${NAME}/g, file);
        }
        return str;
    }
    static renderExportPart(files = []) {
        let str = '';
        for (const file of files) {
            str += '  ' + file + ',\n';
        }
        return FileLinker.exportTemplate.replace(/\${EXPORTS}/g, str);
    }
    link() {
        const links = this.links;
        for (const file in links) {
            if (Object.prototype.hasOwnProperty.call(links, file)) {
                if (file === 'index') {
                    this.shouldLink = !!links[file];
                }
                else if (typeof links[file] === 'string') {
                    const fileName = file;
                    const pathName = `${this.path}/${file}`;
                    this.files.push(fileName);
                    this.paths.push(pathName);
                }
                else {
                    new FileLinker(Object.assign({}, this.config, { path: `${this.path}/${file}`, links: this.links[file] })).link();
                }
            }
        }
        if (this.files.length) {
            this.linkFiles();
        }
        if (this.shouldLink) {
            this.linkIndex();
        }
    }
    linkIndex() {
        const importContent = FileLinker.renderImportPart(this.files);
        const exportContent = FileLinker.renderExportPart(this.files);
        const indexContent = `${importContent}\n${exportContent}`;
        const fullFilePath = (path.resolve(this.path, `index.${this.indexExtension}`));
        if (!fs.existsSync(this.path)) {
            fse.mkdirsSync(this.path);
        }
        fs.writeFileSync(fullFilePath, indexContent);
    }
    linkFiles() {
        for (const fileName of this.files) {
            const extension = this.links[fileName].split('.')[1] || this.extension;
            const templateName = this.links[fileName].split('.')[0];
            const fullFilePath = (path.resolve(this.path, `${fileName}.${extension}`));
            if (!fs.existsSync(this.path)) {
                fse.mkdirsSync(this.path);
            }
            if (!fs.existsSync(fullFilePath)) {
                fs.writeFileSync(fullFilePath, this.renderTemplate(templateName, fileName));
            }
        }
    }
    renderTemplate(templateName, name) {
        return this.templates[templateName].replace(/\${NAME}/g, name) || '';
    }
}
exports.default = FileLinker;
//# sourceMappingURL=FileLinker.js.map