"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Promise = require("bluebird");
const _ = require("lodash");
const factory_1 = require("../factory");
const repository_1 = require("../repository");
const request_1 = require("../request");
const engines_1 = require("./engines");
const util_1 = require("./util");
class ApiRequest {
    constructor(namespace, data) {
        this.request = ApiRequest.start(namespace, data);
    }
    static config(config) {
        if (!config) {
            return;
        }
        const { data, useFactoryOnProduction, factory, } = config;
        if (data) {
            ApiRequest.api.load(data);
        }
        const defaults = ApiRequest.defaults;
        if (useFactoryOnProduction) {
            defaults.useFactoryOnProduction = true;
        }
        if (factory) {
            defaults.factory = factory;
        }
    }
    static start(namespace, data = {}) {
        const { defaults } = ApiRequest;
        if ((defaults.useFactoryOnProduction || process.env.NODE_ENV !== 'production')
            && factory_1.factory.has(namespace)) {
            return new Promise((resolve => resolve(factory_1.factory.make(namespace))));
        }
        const config = _.get(ApiRequest.api.all(), namespace);
        util_1.invariant(config, `The api that you are trying to access is NOT exists: ${namespace}`);
        const _a = new engines_1.APIEntity(config).toObject(), { url } = _a, restConfig = __rest(_a, ["url"]);
        return new request_1.Request(url, data, Object.assign({}, restConfig));
    }
    then(...params) {
        return this.request.then(...params);
    }
    catch(...params) {
        return this.request.catch(...params);
    }
}
ApiRequest.defaults = {
    factory: factory_1.factory,
    useFactoryOnProduction: false,
};
ApiRequest.api = new repository_1.RepositoryLite();
exports.default = ApiRequest;
//# sourceMappingURL=ApiRequest.js.map