"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const storage_1 = require("../../storage");
class Cache {
    constructor(configs = {}) {
        this.storage = storage_1.Storage;
        this.store = {};
        this.stores = {};
        this.list = [];
        const { storeName = 'cache-', } = configs;
        this.storeName = storeName;
        this.configs = configs;
    }
    get length() {
        return this.list.length;
    }
    useStore(storeName) {
        const store = this.stores[storeName];
        if (store) {
            return store;
        }
        this.stores[storeName] = new Cache(Object.assign({}, this.configs, { storeName }));
        return this.stores[storeName];
    }
    get(key, defaultValue) {
        const storageKeyName = this.getItemKeyName(key);
        if (!this.list.includes(storageKeyName)) {
            return defaultValue;
        }
        return this.store[storageKeyName];
    }
    set(key, value, time) {
        const storageKeyName = this.getItemKeyName(key);
        if (!this.list.includes(storageKeyName)) {
            this.list.push(storageKeyName);
        }
        this.store[storageKeyName] = value;
        return Promise.all([
            this.storage.set(this.getListKeyName(), this.list),
            this.storage.set(storageKeyName, value),
        ]).then(() => value);
    }
    remove(key) {
        const storageKeyName = this.getItemKeyName(key);
        const removedData = this.store[storageKeyName];
        _.unset(this.store, storageKeyName);
        _.pull(this.list, storageKeyName);
        return Promise.all([
            this.storage.set(this.getListKeyName(), this.list),
            this.storage.remove(storageKeyName),
        ]).then(() => removedData);
    }
    clear() {
        const removedData = this.all();
        const list = this.list;
        this.list = [];
        this.store = {};
        return Promise.all([
            this.storage.remove(list),
            this.storage.remove(this.getListKeyName()),
        ]).then(() => removedData);
    }
    keys() {
        return this.list.map(key => key.replace(this.storeName, ''));
    }
    all() {
        const output = {};
        for (const name in this.store) {
            if (Object.prototype.hasOwnProperty.call(this.store, name)) {
                output[name.replace(this.storeName, '')] = this.store[name];
            }
        }
        return output;
    }
    merge(object) {
        const returnValue = Object.assign(this.store, object);
        return returnValue;
    }
    restore() {
        return __awaiter(this, void 0, void 0, function* () {
            const listKey = this.getListKeyName();
            this.list = yield this.storage.get(listKey, []);
            this.store = (yield this.storage.get(this.list));
        });
    }
    getListKeyName() {
        return `${this.storeName}list`;
    }
    getItemKeyName(key) {
        return `${this.storeName}${key}`;
    }
}
exports.default = Cache;
//# sourceMappingURL=Cache.js.map