"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class CountDown {
    constructor(config) {
        const { time, tickInterval = 1000, tickImmediatelyOnStart = true, } = config;
        this.time = +time;
        this.tickInterval = tickInterval;
        this.elapseTime = 0;
        this.tickImmediatelyOnStart = tickImmediatelyOnStart;
    }
    start() {
        this.startTime = +new Date();
        this.elapseTime = 0;
        this.tickClock = setInterval(() => {
            return this.handleTick && this.handleTick(this);
        }, this.tickInterval);
        this.endTimer = setTimeout(() => {
            if (this.tickClock) {
                clearInterval(this.tickClock);
                this.tickClock = undefined;
            }
            return this.handleFinish && this.handleFinish(this);
        }, this.time);
        if (this.tickImmediatelyOnStart) {
            this.handleTick && this.handleTick(this);
        }
        return this;
    }
    stop() {
        if (this.tickClock) {
            clearInterval(this.tickClock);
            this.tickClock = undefined;
        }
        if (this.endTimer) {
            clearTimeout(this.endTimer);
            this.endTimer = undefined;
        }
        return this;
    }
    pause() {
        console.error('Not Implemented yet');
        return this;
    }
    resume() {
        console.error('Not Implemented yet');
        return this;
    }
    reset() {
        return this;
    }
    restart() {
        return this.stop().start();
    }
    onTick(callback) {
        this.handleTick = callback;
        return this;
    }
    onFinish(callback) {
        this.handleFinish = callback;
        return this;
    }
    getTime() {
        if (!this.startTime) {
            const time = this.time - this.elapseTime;
            return time > 0 ? time : 0;
        }
        const now = +new Date();
        const currentElapse = now - this.startTime;
        const time = this.time - this.elapseTime - currentElapse;
        return time > 0 ? time : 0;
    }
    getHour() {
        return Math.floor(this.getTime() / 360000) % 24;
    }
    getMinute() {
        return Math.floor(this.getTime() / 60000) % 60;
    }
    getSecond() {
        return Math.floor(this.getTime() / 1000) % 60;
    }
    getMillisecond() {
        return this.getTime() % 1000;
    }
    toHours() {
        return Math.floor(this.getTime() / 360000);
    }
    toMinutes() {
        return Math.floor(this.getTime() / 60000);
    }
    toSeconds() {
        return Math.floor(this.getTime() / 1000);
    }
    toMilliseconds() {
        return this.getTime();
    }
}
exports.default = CountDown;
//# sourceMappingURL=CountDown.js.map