"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const shared_1 = require("../shared");
class Timer {
    constructor(config = shared_1.EMPTY_OBJECT) {
        this.status = 'stand-by';
        const { tickInterval = 1000, tickImmediatelyOnStart = true, } = config;
        this.accumulatedTime = 0;
        this.tickInterval = tickInterval;
        this.tickImmediatelyOnStart = tickImmediatelyOnStart;
    }
    start() {
        this.accumulatedTime = 0;
        this.startTime = new Date();
        this.tickClock = setInterval(() => {
            return this.handleTick && this.handleTick(this);
        }, this.tickInterval);
        if (this.tickImmediatelyOnStart) {
            this.handleTick && this.handleTick(this);
        }
        return this;
    }
    stop() {
        const stopTime = new Date();
        const startTime = this.startTime;
        if (!startTime) {
            return this;
        }
        this.accumulatedTime += +stopTime - +startTime;
        this.startTime = undefined;
        if (this.tickClock) {
            clearInterval(this.tickClock);
            this.tickClock = undefined;
        }
        return this;
    }
    restart() {
        return this.stop().start();
    }
    pause() {
        const pauseTime = new Date();
        const startTime = this.startTime;
        if (!startTime) {
            return this;
        }
        this.accumulatedTime += +pauseTime - +startTime;
        this.startTime = undefined;
        clearInterval(this.tickClock);
        return this;
    }
    resume() {
        if (!this.accumulatedTime || this.startTime) {
            return this;
        }
        this.startTime = new Date();
        return this;
    }
    onTick(callback) {
        this.handleTick = callback;
        return this;
    }
    getTime() {
        if (!this.startTime) {
            return +this.accumulatedTime;
        }
        const now = new Date();
        return +now - +this.startTime + this.accumulatedTime;
    }
    getHour() {
        return Math.floor(this.getTime() / 360000) % 24;
    }
    getMinute() {
        return Math.floor(this.getTime() / 60000) % 60;
    }
    getSecond() {
        return Math.floor(this.getTime() / 1000) % 60;
    }
    getMillisecond() {
        return this.getTime() % 1000;
    }
    toHours() {
        return Math.floor(this.getTime() / 360000);
    }
    toMinutes() {
        return Math.floor(this.getTime() / 60000);
    }
    toSeconds() {
        return Math.floor(this.getTime() / 1000);
    }
    toMilliseconds() {
        return this.getTime();
    }
}
exports.default = Timer;
//# sourceMappingURL=Timer.js.map