"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Abstractions_1 = require("rua-core/lib/Abstractions");
class RuaDva extends Abstractions_1.AbstractRuaPackage {
    constructor() {
        super(...arguments);
        this.store = {};
        this.actions = {};
        this.dispatch = (...args) => this.store._store.dispatch(...args);
        this.oldModelPath = '__rua_model';
        this.oldUnmodelPath = '__rua_unmodel';
        this.oldStartPath = '__rua_start';
    }
    static validateDva(dva) {
    }
    saveDispatch(dispatch) {
        this.dispatch = dispatch;
    }
    rua(dva) {
        RuaDva.validateDva(dva);
        this.saveDva(dva);
        this.registerExistingModels();
        this.interceptModel();
        this.interceptUnmodel();
        this.interceptStart();
        this.booted = true;
        return this.store;
    }
    saveDva(dva) {
        this.store = dva;
        this.store[this.oldModelPath] = undefined;
        this.store[this.oldUnmodelPath] = undefined;
        this.store[this.oldStartPath] = undefined;
    }
    registerExistingModels() {
        const models = this.store._models;
        for (const model in models) {
            if (Object.prototype.hasOwnProperty.call(models, model)) {
                this.registerModel(models[model]);
            }
        }
    }
    registerModel(model) {
        const { namespace, reducers, effects } = model;
        this.actions[namespace] = Object.assign({}, this.registerActions(namespace, reducers), this.registerActions(namespace, effects));
    }
    registerActions(namespace, reducers) {
        const actions = {};
        for (const reducer in reducers) {
            if (Object.prototype.hasOwnProperty.call(reducers, reducer)) {
                actions[reducer] = (payload, extra) => {
                    if (!extra) {
                        return this.dispatch({
                            payload,
                            type: `${namespace}/${reducer}`,
                        });
                    }
                    return this.dispatch({
                        payload,
                        extra,
                        type: `${namespace}/${reducer}`,
                    });
                };
            }
        }
        return actions;
    }
    unregisterActions(namespace) {
        delete this.actions[namespace];
        return !this.actions[namespace];
    }
    interceptModel() {
        this.store[this.oldModelPath] = this.store.model;
        this.store.model = (model) => {
            this.registerModel(model);
            return this.store[this.oldModelPath](model);
        };
    }
    interceptUnmodel() {
        this.store[this.oldUnmodelPath] = this.store.unmodel;
        this.store.unmodel = (model) => {
            const originalOutput = this.store.__rua_unmodel(model);
            this.unregisterActions(model);
            return originalOutput;
        };
    }
    interceptStart() {
        this.store[this.oldStartPath] = this.store.start;
        this.store.start = (...params) => {
            const originalOutput = this.store[this.oldStartPath](...params);
            this.saveDispatch(this.store._store.dispatch);
            return originalOutput;
        };
    }
}
exports.default = RuaDva;
//# sourceMappingURL=RuaDva.js.map