"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const memory_1 = require("../../memory");
const util_1 = require("../../request/util");
const Immutable = require("seamless-immutable");
const prefix = 'dva-model-backup-';
function assignState(state, action) {
    const { payload } = action;
    if (Immutable.isImmutable(state) && !Immutable.isImmutable(payload)) {
        return Immutable(payload);
    }
    return payload;
}
function setState(state, action) {
    const { payload } = action;
    if (Immutable.isImmutable(state)) {
        return state.merge(payload);
    }
    return Object.assign({}, state, payload);
}
function mergeState(state, action) {
    const { payload } = action;
    if (Immutable.isImmutable(state)) {
        return state.merge(payload, { deep: true });
    }
    const outState = Object.assign({}, _.merge(state, action.payload));
    for (const key in action.payload) {
        const outValue = outState[key];
        if (Array.isArray(outValue)) {
            outState[key] = [...outValue];
        }
        else if (util_1.isPlainObject(outValue)) {
            outState[key] = Object.assign({}, outValue);
        }
    }
    return outState;
}
function clearState(state, action) {
    if (Immutable.isImmutable(state)) {
        return Immutable({});
    }
    return {};
}
function backupState(state, action) {
    const namespace = action.type.split('/')[0];
    if (Immutable.isImmutable(state)) {
        memory_1.Memory.set(`${prefix}${namespace}`, state);
    }
    else {
        memory_1.Memory.set(`${prefix}${namespace}`, _.clone(state));
    }
    return state;
}
function rollbackState(state, action) {
    const namespace = action.type.split('/')[0];
    if (Immutable.isImmutable(state)) {
        return memory_1.Memory.get(`${prefix}${namespace}`);
    }
    return _.clone(memory_1.Memory.get(`${prefix}${namespace}`));
}
function dvaReducerGenerator(defaultState) {
    const _defaultState = defaultState;
    function resetState(state, actions) {
        const { payload: key } = actions;
        if (!key) {
            return Object.assign({}, state, _defaultState());
        }
        if (Array.isArray(key)) {
            const otherState = {};
            const defaultState = _defaultState();
            key.forEach((k) => {
                otherState[k] = defaultState[k];
            });
            return Object.assign(state, otherState);
        }
        return Object.assign(state, {
            [key]: _defaultState()[key],
        });
    }
    return {
        setState,
        resetState,
        mergeState,
        assignState,
        clearState,
        backupState,
        rollbackState,
    };
}
exports.default = dvaReducerGenerator;
//# sourceMappingURL=dvaReducerGenerator.js.map