"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const FunctionCollectionDescriptorBuildUtil_1 = require("../../utility/FunctionCollectionDescriptorBuildUtil");
function IncreaseState(stateKeyOrConfig) {
    return function (target, key, descriptor) {
        if (target[key]) {
            return console.error('[Decorator]IncreaseState will override original function');
        }
        return FunctionCollectionDescriptorBuildUtil_1.default.create(target, key, descriptor, function () {
            if ('string' === typeof stateKeyOrConfig) {
                return this.setState((state) => {
                    return {
                        [stateKeyOrConfig]: state[stateKeyOrConfig] + 1,
                    };
                });
            }
            const { key, step = 1, max, } = stateKeyOrConfig;
            return this.setState((state) => {
                const currentValue = state[key];
                if (undefined !== max && currentValue > max) {
                    return {
                        [key]: max,
                    };
                }
                let newValue = state[key] + step;
                if (undefined !== max && newValue > max) {
                    newValue = max;
                }
                return {
                    [key]: newValue,
                };
            });
        });
    };
}
exports.default = IncreaseState;
//# sourceMappingURL=IncreaseState.js.map