"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const shared_1 = require("../shared");
const engines_1 = require("./engines");
const internals_1 = require("./internals");
class Request {
    constructor(url, bodyOrQuery, options = {}) {
        const { headers, } = options;
        this.headers = new internals_1.Header(headers);
        this.url = new internals_1.Url(url);
        this.configuration = Object.assign({}, options);
        const method = (options.method || Request.defaults.method).toUpperCase();
        this.configuration.method = method;
        if (method
            && (method === 'POST'
                || method === 'PUT'
                || method === 'PATCH')) {
            this.body = new internals_1.Body(Object.assign({}, this.configuration.body, bodyOrQuery));
        }
        else {
            this.body = new internals_1.Body();
            this.configuration.query = Object.assign({}, this.configuration.query, bodyOrQuery);
        }
        this.request = this.start();
    }
    static config(requestConfig) {
        const { defaultMethod, requestInterceptors = shared_1.EMPTY_OBJECT, responseInterceptors = shared_1.EMPTY_OBJECT, } = requestConfig;
        if (defaultMethod) {
            Request.defaults.method = defaultMethod;
        }
        for (const key in requestInterceptors) {
            Request.interceptors.request.push(requestInterceptors[key]);
        }
        for (const key in responseInterceptors) {
            Request.interceptors.response.push(responseInterceptors[key]);
        }
    }
    start() {
        const requestInterceptors = Request.interceptors.request;
        const responseInterceptors = Request.interceptors.response;
        const providedInterceptor = Request.interceptor;
        requestInterceptors.forEach((interceptor) => {
            if (typeof interceptor === 'string') {
                return providedInterceptor[interceptor](this);
            }
            return interceptor(this);
        });
        return Request.defaults.engine(Object.assign({}, this.configuration, { url: this.url, headers: this.headers, body: this.body, after: (response) => {
                responseInterceptors.forEach((interceptor) => {
                    if (typeof interceptor === 'string') {
                        return providedInterceptor[interceptor](response);
                    }
                    return interceptor(response);
                });
                const after = this.configuration.after;
                if (after && 'function' === typeof after) {
                    after(response);
                }
                return response;
            } }));
    }
    then(...params) {
        return this.request.then(...params);
    }
    catch(...params) {
        return this.request.catch(...params);
    }
}
Request.interceptors = {
    request: [],
    response: [],
};
Request.defaults = {
    engine: engines_1.superAgentEngine,
    method: 'GET',
};
Request.engines = {
    superAgent: engines_1.superAgentEngine,
    fetch: engines_1.fetchEngine,
};
Request.interceptor = {};
exports.default = Request;
//# sourceMappingURL=Request.js.map