"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("superagent");
const exception_1 = require("../../exception");
function superAgentEngine(options) {
    const { url, headers, method, query, type, accept, withCredentials, timeout, retry, retryCallback, before, after, form, body, } = options;
    let req = request(method, url.toString())
        .set(headers.getHeaders());
    if (form) {
        req = req.field(body.toObject());
    }
    else {
        req = req.send(body.toObject());
    }
    type && (req = req.type(type));
    query && (req = req.query(query));
    accept && (req = req.accept(accept));
    withCredentials && (req = req.withCredentials());
    timeout && (req = req.timeout(timeout));
    retry && (req = req.retry(retry, retryCallback));
    before && before(req);
    return req
        .then(response => after(response))
        .then((response) => {
        if (response.status >= 200 && response.status < 300) {
            return response;
        }
        const err = new exception_1.HttpException(response.status, response.type);
        err.response = response;
        throw err;
    })
        .then(response => response.body)
        .catch((err) => {
        if (err.message.includes('Timeout')) {
            throw new exception_1.HttpRequestTimeoutException();
        }
        if ('Not Found' === err.message) {
            throw new exception_1.HttpNotFoundException();
        }
        throw err;
    });
}
exports.default = superAgentEngine;
//# sourceMappingURL=superAgentEngine.js.map