"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dev = process.env.NODE_ENV !== 'production';
const { hasOwnProperty } = {};
const { slice } = [];
function warn(msg) {
    if (typeof console !== 'undefined' && dev) {
        console.warn('[RuaRequest warn]: ' + msg);
    }
}
exports.warn = warn;
function error(msg) {
    if (typeof console !== 'undefined') {
        console.error(msg);
    }
}
exports.error = error;
function trim(str) {
    return str ? str.replace(/^\s*|\s*$/g, '') : '';
}
exports.trim = trim;
function trimEnd(str, chars) {
    if (str && chars === undefined) {
        return str.replace(/\s+$/, '');
    }
    if (!str || !chars) {
        return str;
    }
    return str.replace(new RegExp(`[${chars}]+$`), '');
}
exports.trimEnd = trimEnd;
function toLower(str) {
    return str ? str.toLowerCase() : '';
}
exports.toLower = toLower;
function toUpper(str) {
    return str ? str.toUpperCase() : '';
}
exports.toUpper = toUpper;
exports.isArray = Array.isArray;
function isString(val) {
    return typeof val === 'string';
}
exports.isString = isString;
function isBoolean(val) {
    return val === true || val === false;
}
exports.isBoolean = isBoolean;
function isFunction(val) {
    return typeof val === 'function';
}
exports.isFunction = isFunction;
function isObject(obj) {
    return obj !== null && typeof obj === 'object';
}
exports.isObject = isObject;
function isPlainObject(obj) {
    return isObject(obj) && Object.getPrototypeOf(obj) == Object.prototype;
}
exports.isPlainObject = isPlainObject;
function isBlob(obj) {
    return typeof Blob !== 'undefined' && obj instanceof Blob;
}
exports.isBlob = isBlob;
function isFormData(obj) {
    return typeof FormData !== 'undefined' && obj instanceof FormData;
}
exports.isFormData = isFormData;
function each(obj, iterator) {
    let i;
    let key;
    if (exports.isArray(obj)) {
        for (i = 0; i < obj.length; i += 1) {
            iterator.call(obj[i], obj[i], i);
        }
    }
    else if (isObject(obj)) {
        for (key in obj) {
            if (hasOwnProperty.call(obj, key)) {
                iterator.call(obj[key], obj[key], key);
            }
        }
    }
    return obj;
}
exports.each = each;
//# sourceMappingURL=util.js.map