"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const localForage = require("localforage");
class StorageEngine {
    constructor() {
        localForage.config();
    }
    get length() {
        return localForage.length();
    }
    set(key, value) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(key)) {
                const keys = key;
                for (const index in keys) {
                    yield localForage.setItem(keys[index], value[index]);
                }
                return;
            }
            yield localForage.setItem(key, value);
            return;
        });
    }
    get(key, defaultValue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(key)) {
                const keys = key;
                const output = {};
                for (const index in keys) {
                    output[keys[index]] = yield localForage.getItem(keys[index]);
                }
                return output;
            }
            const value = yield localForage.getItem(key);
            return value || defaultValue;
        });
    }
    remove(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(key)) {
                const keys = key;
                for (const index in keys) {
                    yield localForage.removeItem(keys[index]);
                }
                return;
            }
            return localForage.removeItem(key);
        });
    }
    clear() {
        return __awaiter(this, void 0, void 0, function* () {
            return localForage.clear();
        });
    }
    keys() {
        return __awaiter(this, void 0, void 0, function* () {
            return localForage.keys();
        });
    }
    values() {
        return __awaiter(this, void 0, void 0, function* () {
            const keys = yield this.keys();
            return this.get(keys);
        });
    }
    all() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = {};
            const keys = yield this.keys();
            for (const key of keys) {
                data[key] = yield this.get(key);
            }
            return data;
        });
    }
}
exports.default = StorageEngine;
//# sourceMappingURL=StorageEngine.js.map