"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_native_1 = require("react-native");
const _ = require("lodash");
class StorageEngine {
    get length() {
        return this.keys().then(keys => keys.length);
    }
    set(key, value) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(key)) {
                const values = value
                    .map((item) => JSON.stringify(item));
                const pair = _.zip(key, values);
                yield react_native_1.AsyncStorage.multiSet(pair);
                return;
            }
            yield react_native_1.AsyncStorage.setItem(key, JSON.stringify(value));
            return;
        });
    }
    get(key, defaultValue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(key)) {
                const data = yield react_native_1.AsyncStorage.multiGet(key);
                data.map((item) => {
                    return [item[0], JSON.parse(item[1])];
                });
                return data.length ? _.fromPairs(data) : defaultValue;
            }
            const value = yield react_native_1.AsyncStorage.getItem(key);
            return JSON.parse(value) || defaultValue;
        });
    }
    remove(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(key)) {
                return yield react_native_1.AsyncStorage.multiRemove(key);
            }
            return yield react_native_1.AsyncStorage.removeItem(key);
        });
    }
    clear() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield react_native_1.AsyncStorage.clear();
        });
    }
    keys() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield react_native_1.AsyncStorage.getAllKeys();
        });
    }
    values() {
        return __awaiter(this, void 0, void 0, function* () {
            const keys = yield this.keys();
            return this.get(keys);
        });
    }
    all() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = {};
            const keys = yield this.keys();
            const rawAllData = yield react_native_1.AsyncStorage.multiGet(keys);
            for (const rawItem in rawAllData) {
                data[rawItem[0]] = rawItem[1];
            }
            return data;
        });
    }
}
exports.default = StorageEngine;
//# sourceMappingURL=StorageEngine.native.js.map