"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class RuleValidationEngine {
    constructor() {
        this.validators = {};
    }
    setValidators(validators) {
        this.validators = validators;
    }
    hasValidator(rules) {
        return rules.all().reduce((result, rule) => {
            const validatorName = rule.name;
            return result && !!this.validators[validatorName];
        }, true);
    }
    getUnregisteredValidatorName(rules) {
        return rules.all().reduce((carry, rule) => {
            const validatorName = rule.name;
            if (!this.validators[validatorName]) {
                carry.push(validatorName);
            }
            return carry;
        }, []);
    }
    validate(data, rules) {
        return rules.all().reduce((carry, rule) => {
            const validatorName = rule.name;
            return this.validators[validatorName].call(this, carry, data, ...rule.args);
        }, true);
    }
}
exports.default = RuleValidationEngine;
//# sourceMappingURL=RuleValidationEngine.js.map