"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Rules_1 = require("./Rules");
const DataConvertorEngine_1 = require("./DataConvertorEngine");
const RuleValidationEngine_1 = require("./RuleValidationEngine");
class ValidatorEngine {
    constructor(config) {
        this.failedRules = [];
        this.failedMessages = [];
        this.unregisteredRules = [];
        this.hasUnregisteredRule = false;
        this.ruleValidator = new RuleValidationEngine_1.default();
        this.dataTranslator = new DataConvertorEngine_1.default();
        this.rules = {};
        this.data = {};
        this.isValid = true;
        this.isFixed = false;
        this.config(config);
    }
    config(config) {
        const { data, rules, validators, extraValidators, } = config;
        this.data = data;
        this.rules = rules;
        this.ruleValidator.setValidators(Object.assign({}, validators, extraValidators));
    }
    validate() {
        for (const ruleKey in this.rules) {
            const ruleString = this.rules[ruleKey];
            if (!ruleString) {
                continue;
            }
            const data = this.data[ruleKey];
            const rule = new Rules_1.default(ruleString);
            if (!this.ruleValidator.hasValidator(rule)) {
                this.hasUnregisteredRule = true;
                continue;
            }
            this.isValid = this.isValid && this.ruleValidator.validate(data, rule);
        }
    }
    fails() {
        this.validate();
        return !this.isValid && !this.isFixed;
    }
    passes() {
        this.validate();
        return this.isValid || this.isFixed;
    }
}
exports.default = ValidatorEngine;
//# sourceMappingURL=ValidatorEngine.js.map